@@ -11,7 +11,11 @@
 #'     `make.names(unique = FALSE, allow_ = TRUE)` to ensure agreement
 #'     between the data and the output of `brms`.
 #'   * For each implicitly missing outcome observation, add explicit row
-#'     with the outcome variable equal to `NA_real_`.
+#'     with the outcome variable equal to `NA_real_`. Missing values
+#'     in the predictors are implicitly filled using [zoo::na.locf()]
+#'     on within each patient, which is not valid for time-varying
+#'     covariates. If any covariates are time-varying, please
+#'     manually perform this step before calling [brm_data()].
 #'   * Arrange the rows of the data by group, then patient, then discrete time.
 #'   * Select only the columns of the data relevant to an MMRM analysis.
 #' @section Separation string:
@@ -24,57 +28,94 @@
 #' @return A classed tibble with attributes which denote features of
 #'   the data such as the treatment group and discrete time variables.
 #' @param data Data frame or tibble with longitudinal data.
-#' @param outcome Character of length 1, name of the outcome variable.
-#' @param role Character of length 1. Either `"response"` if `outcome`
-#'   is the raw response variable (e.g. AVAL) or `"change"` if `outcome`
-#'   is change from baseline (e.g. CHG).
-#' @param group Character of length 1, name of the treatment group variable.
-#'   Must point to a character vector in the data. Factors are converted
-#'   to characters.
-#' @param subgroup Character of length 1, optional name of the a
-#'   discrete subgroup variable. Set to `NULL` to omit the subgroup (default).
+#' @param outcome Character of length 1, name of the continuous
+#'   outcome variable.
+#'   Example possibilities from clinical trial datasets include
+#'   `"CHG"` and `"AVAL"`.
+#'   The `outcome` column in the data should be a numeric vector.
 #' @param baseline Character of length 1,
-#'   name of the baseline response variable.
+#'   name of the baseline response variable (for example, `"BASE"`
+#'   in many clinical trial datasets).
 #'   Only relevant if the response variable is change from baseline.
 #'   Supply `NULL` to ignore or omit.
+#' @param group Character of length 1, name of the treatment group variable.
+#'   Example possibilities from clinical trial datasets include
+#'   `"TRT01P"`, `"TREATMENT"`, `"TRT"`, and `"GROUP"`.
+#'   The `group` column in the data should be a
+#'   character vector or unordered factor.
+#' @param subgroup Character of length 1, optional name of the a
+#'   discrete subgroup variable. Set to `NULL` to omit the subgroup (default).
+#'   If present, the `subgroup` column in the data should be a
+#'   character vector or unordered factor.
 #' @param time Character of length 1, name of the discrete time variable.
-#'   Must point to a character vector in the data. Factors are converted
-#'   to characters.
+#'   Example possibilities from clinical trial datasets include
+#'   `"AVISIT"` and `"VISIT"`.
+#'   For most analyses, please ensure the time column in the data
+#'   is an ordered factor. You can easily turn
+#'   the time variable into an ordered factor using
+#'   [brm_data_chronologize()], either before or immediately after
+#'   [brm_data()] (but before any `brm_archetype_*()` functions).
+#'   This ensures the time points sort in chronological order,
+#'   which ensures the correctness of informative prior archetypes and
+#'   autoregressive / moving average correlation structures.
+#'
+#'   Ordinarily, ordered factors automatically use polynomial contrasts from
+#'   [contr.poly()]. This is undesirable for MMRMs, so if the time variable
+#'   is an ordered factor, then [brm_data()]
+#'   manually sets `contrasts(data[[time]])` to a set of treatment contrasts
+#'   using [contr.treatment()]. If you prefer different contrasts, please
+#'   manually set `contrasts(data[[time]])` to something else after
+#'   calling [brm_data()].
 #' @param patient Character of length 1, name of the patient ID variable.
+#'   Example possibilities from clinical trial datasets include
+#'   `"USUBJID"`, `"SUBJID"`, `"PATIENT"`, `"PATIENTID"`, `"SUBJECT"`,
+#'   `"SUBJIDID"`, `"SBJID"`, `"STYSID1A"`, `"SBJ1N"`, and `"ID"`.
+#'   The `patient` column in the data should be a factor or character vector.
 #' @param covariates Character vector of names of other covariates.
+#'   All these covariates are assumed to be non-time-varying. For time-varying
+#'   covariates, please manually expand the data to the full grid of patients
+#'   and time points before you call [brm_data()]. See the "Preprocessing"
+#'   section for details.
 #' @param missing Character of length 1, name of an optional variable
 #'   in a simulated dataset to indicate which outcome values should be missing.
 #'   Set to `NULL` to omit.
-#' @param reference_group Character of length 1, Level of the `group` column
-#'   to indicate the control group.
+#' @param reference_group Atomic value of length 1, Level of the `group`
+#'   column to indicate the control group.
+#'   Example possibilities from clinical trial datasets include
+#'   `"Placebo"`, `"PLACEBO"`, `"PBO"`, `"PLB"`, `"CONTROL"`, `"CTRL"`,
+#'   `"REFERENCE"`, and `"REF"`.
 #'   `reference_group` only applies to the post-processing that happens
 #'   in functions like [brm_marginal_draws()] downstream of the model.
 #'   It does not control the fixed effect mapping in the
 #'   model matrix that `brms` derives from the formula from `brm_formula()`.
-#' @param level_control Deprecated on 2024-01-11 (version 0.2.0.9002).
-#'   Use `reference_group` instead.
-#' @param reference_subgroup Character of length 1,
+#' @param reference_subgroup Atomic value of length 1,
 #'   level of the `subgroup` column
 #'   to use as a reference for pairwise differences in when computing
 #'   marginal means downstream of the model.
 #'   It does not control the fixed effect mapping in the
 #'   model matrix that `brms` derives from the formula from `brm_formula()`.
-#' @param reference_time Character of length 1 or `NULL`,
+#' @param reference_time Atomic value of length 1 or `NULL`,
 #'   level of the `time` column to indicate the baseline time point.
-#'   This value should not be present in the data if the outcome
-#'   variable is change from baseline, but it must be in the data
-#'   if the outcome variable is the raw response so that `brms.mmrm`
-#'   can produce model-based marginal estimates of change from baseline.
-#'   In other words, set `reference_time` to `NULL` if `role` is `"change"`,
-#'   and set `reference_time` to a non-null value in `data[[time]]`
-#'   if `role` is `"response"`.
+#'   Leave as `NULL` if there is no baseline or baseline is not included
+#'   in `data[[time]]`.
+#'
+#'   If `reference_time` is not `NULL`, then [brm_marginal_draws()] will
+#'   calculate change from baseline, and it will calculate treatment
+#'   differences as differences between change-from-baseline values.
+#'   If `reference_time` is not `NULL`, then [brm_marginal_draws()] will
+#'   not calculate change from baseline, and it will calculate treatment
+#'   differences as differences between response values.
 #'
 #'   Note: `reference_time` only applies to the post-processing that happens
 #'   in functions like [brm_marginal_draws()] downstream of the model.
 #'   It does not control the fixed effect mapping in the
 #'   model matrix that `brms` derives from the formula from `brm_formula()`.
+#' @param role Deprecated as unnecessary on 2024-07-11 (version 1.0.1.9007).
+#'   Use `reference_time` to supply a baseline time point value if it exists.
 #' @param level_baseline Deprecated on 2024-01-11 (version 0.2.0.9002).
 #'   Use `reference_time` instead.
+#' @param level_control Deprecated on 2024-01-11 (version 0.2.0.9002).
+#'   Use `reference_group` instead.
 #' @examples
 #' set.seed(0)
 #' data <- brm_simulate_simple()$data
@@ -83,7 +124,6 @@
 #' brm_data(
 #'   data = data,
 #'   outcome = "col_response",
-#'   role = "response",
 #'   group = "col_group",
 #'   time = "col_time",
 #'   patient = "col_patient",
@@ -92,22 +132,29 @@
 #' )
 brm_data <- function(
   data,
-  outcome = "CHG",
-  role = "change",
+  outcome,
   baseline = NULL,
-  group = "TRT01P",
+  group,
   subgroup = NULL,
-  time = "AVISIT",
-  patient = "USUBJID",
+  time,
+  patient,
   covariates = character(0L),
   missing = NULL,
-  reference_group = "Placebo",
-  level_control = NULL,
+  reference_group,
   reference_subgroup = NULL,
   reference_time = NULL,
-  level_baseline = NULL
+  role = NULL,
+  level_baseline = NULL,
+  level_control = NULL
 ) {
   assert(is.data.frame(data), message = "data arg must be a data frame.")
+  if (!is.null(role)) {
+    brm_deprecate(
+      "The 'role' argument was deprecated as unnecessary on 2024-07-11 ",
+      "(version 1.0.1.9007). Use reference_time to specify a baseline ",
+      "time value if it exists."
+    )
+  }
   if (!is.null(level_control)) {
     brm_deprecate(
       "level_control was deprecated on 2024-01-11 (version 0.2.0.9002). ",
@@ -127,7 +174,6 @@ brm_data <- function(
   out <- brm_data_new(
     data = data,
     brm_outcome = as.character(outcome),
-    brm_role = as.character(role),
     brm_baseline = baseline,
     brm_group = as.character(group),
     brm_subgroup = subgroup,
@@ -135,7 +181,7 @@ brm_data <- function(
     brm_patient = as.character(patient),
     brm_covariates = as.character(covariates),
     brm_missing = missing,
-    brm_reference_group = as.character(reference_group),
+    brm_reference_group = reference_group,
     brm_reference_subgroup = reference_subgroup,
     brm_reference_time = reference_time
   )
@@ -146,7 +192,6 @@ brm_data <- function(
 brm_data_new <- function(
   data,
   brm_outcome = NULL,
-  brm_role = NULL,
   brm_baseline = NULL,
   brm_group = NULL,
   brm_subgroup = NULL,
@@ -156,19 +201,12 @@ brm_data_new <- function(
   brm_missing = NULL,
   brm_reference_group = NULL,
   brm_reference_subgroup = NULL,
-  brm_reference_time = NULL,
-  brm_levels_group = NULL,
-  brm_levels_subgroup = NULL,
-  brm_levels_time = NULL,
-  brm_labels_group = NULL,
-  brm_labels_subgroup = NULL,
-  brm_labels_time = NULL
+  brm_reference_time = NULL
 ) {
   out <- tibble::new_tibble(x = data, class = "brms_mmrm_data")
   structure(
     out,
     brm_outcome = brm_outcome,
-    brm_role = brm_role,
     brm_baseline = brm_baseline,
     brm_group = brm_group,
     brm_subgroup = brm_subgroup,
@@ -178,20 +216,13 @@ brm_data_new <- function(
     brm_missing = brm_missing,
     brm_reference_group = brm_reference_group,
     brm_reference_subgroup = brm_reference_subgroup,
-    brm_reference_time = brm_reference_time,
-    brm_levels_group = brm_levels_group,
-    brm_levels_subgroup = brm_levels_subgroup,
-    brm_levels_time = brm_levels_time,
-    brm_labels_group = brm_labels_group,
-    brm_labels_subgroup = brm_labels_subgroup,
-    brm_labels_time = brm_labels_time
+    brm_reference_time = brm_reference_time
   )
 }
 
 brm_data_preprocess <- function(out) {
-  out <- brm_data_level(out)
   out <- brm_data_fill(out)
-  out <- brm_data_select(out)
+  out <- brm_time_contrasts(out)
   out
 }
 
@@ -202,16 +233,12 @@ brm_data_validate <- function(data) {
 #' @export
 brm_data_validate.default <- function(data) {
   outcome <- attr(data, "brm_outcome")
-  role <- attr(data, "brm_role")
   baseline <- attr(data, "brm_baseline")
   group <- attr(data, "brm_group")
   subgroup <- attr(data, "brm_subgroup")
   time <- attr(data, "brm_time")
   patient <- attr(data, "brm_patient")
   covariates <- attr(data, "brm_covariates")
-  levels_group <- attr(data, "brm_levels_group")
-  levels_subgroup <- attr(data, "brm_levels_subgroup")
-  levels_time <- attr(data, "brm_levels_time")
   missing <- attr(data, "brm_missing")
   reference_group <- attr(data, "brm_reference_group")
   reference_subgroup <- attr(data, "brm_reference_subgroup")
@@ -222,7 +249,6 @@ brm_data_validate.default <- function(data) {
     message = "please use brm_data() to preprocess your data"
   )
   assert_chr(outcome, "outcome of data must be a nonempty character string")
-  assert_chr(role, "role of data must be a nonempty character string")
   assert_chr(
     baseline %|||% "x",
     "baseline must NULL or a nonempty character string"
@@ -239,12 +265,17 @@ brm_data_validate.default <- function(data) {
   assert_chr(reference_group, "reference_group must be a nonempty string")
   assert_chr(
     reference_subgroup %|||% "x",
-    "reference_subgroup must NULL or a nonempty character string"
+    paste(
+      "reference_subgroup must NULL or a nonempty element of the subgroup",
+      "column in the data"
+    )
   )
-  assert_chr(reference_time %|||% "x", "reference_time must NULL or character")
-  assert(
-    role %in% c("response", "change"),
-    message = "role must be either \"response\" or \"change\""
+  assert_chr(
+    reference_time %|||% "x",
+    paste(
+      "reference_time must NULL or a nonempty element of the time column",
+      "in the data"
+    )
   )
   assert_col(outcome, data)
   assert_col(baseline, data)
@@ -263,39 +294,30 @@ brm_data_validate.default <- function(data) {
   assert_machine_names(covariates)
   assert_machine_names(missing)
   assert(
-    all(levels_group %in% data[[group]]),
-    message = "all group levels must be in data[[group]]"
-  )
-  assert(
-    reference_group %in% data[[group]],
-    message = "reference_group must be in data[[group]]"
+    reference_group,
+    !anyNA(.),
+    length(.) == 1L,
+    . %in% data[[group]],
+    message = "reference_group must be an element of data[[group]]"
   )
   if (!is.null(subgroup)) {
-    assert(
-      all(levels_subgroup %in% data[[subgroup]]),
-      message = "all subgroup levels must be in data[[subgroup]]"
-    )
     assert(
       reference_subgroup,
-      is.character(.),
       !anyNA(.),
       length(.) == 1L,
-      nzchar(.),
       . %in% data[[subgroup]],
       message = paste(
-        "reference_subgroup must be a nonempty character string",
-        "in data[[subgroup]]"
+        "reference_subgroup must be an element of data[[subgroup]]"
       )
     )
   }
-  assert(
-    all(levels_time %in% data[[time]]),
-    message = "all time levels must be in data[[time]]"
-  )
   if (!is.null(reference_time)) {
     assert(
-      all(reference_time %in% data[[time]]),
-      message = "reference_time must be in data[[time]]"
+      reference_time,
+      !anyNA(.),
+      length(.) == 1L,
+      . %in% data[[time]],
+      message = "reference_time must be an element of data[[time]]"
     )
   }
   sep <- brm_sep()
@@ -343,112 +365,17 @@ brm_data_validate.default <- function(data) {
   }
   for (column in c(group, subgroup, time)) {
     assert(
-      is.character(data[[column]]) || is.factor(data[[column]]),
+      is.atomic(data[[column]]) || is.factor(data[[column]]),
+      !is.factor(data[[column]]) || !anyNA(levels(data[[column]])),
       message = paste(
         column,
-        "column in the data must be a character vector."
+        paste(
+          "column in the data must be an atomic or factor type,",
+          "and all factor levels must be non-missing."
+        )
       )
     )
   }
-  if (role == "response") {
-    assert(
-      !is.null(reference_time),
-      message = "reference_time is needed if role is \"response\"."
-    )
-  } else if (role == "change") {
-    assert(
-      is.null(reference_time),
-      message = "reference_time should be NULL if role is \"change\"."
-    )
-  }
-}
-
-brm_data_select <- function(data) {
-  columns <- c(
-    attr(data, "brm_outcome"),
-    attr(data, "brm_missing"),
-    attr(data, "brm_baseline"),
-    attr(data, "brm_group"),
-    attr(data, "brm_subgroup"),
-    attr(data, "brm_time"),
-    attr(data, "brm_patient"),
-    attr(data, "brm_covariates")
-  )
-  data[, as.character(unique(columns)), drop = FALSE]
-}
-
-brm_data_level <- function(data) {
-  data <- brm_data_level_group(data)
-  if (length(attr(data, "brm_subgroup"))) {
-    data <- brm_data_level_subgroup(data)
-  }
-  brm_data_level_time(data)
-}
-
-brm_data_level_group <- function(data) {
-  group <- attr(data, "brm_group")
-  reference_group <- attr(data, "brm_reference_group")
-  names_group <- brm_levels(data[[group]])
-  all_group <- tibble::tibble(label = data[[group]], level = names_group)
-  data[[group]] <- as.character(names_group)
-  meta_group <- dplyr::arrange(dplyr::distinct(all_group), level)
-  attr(data, "brm_reference_group") <- brm_levels(reference_group)
-  attr(data, "brm_levels_group") <- sort(as.character(meta_group$level))
-  attr(data, "brm_labels_group") <- sort(as.character(meta_group$label))
-  data
-}
-
-brm_data_level_subgroup <- function(data) {
-  subgroup <- attr(data, "brm_subgroup")
-  reference_subgroup <- attr(data, "brm_reference_subgroup")
-  names_subgroup <- brm_levels(data[[subgroup]])
-  all_subgroup <- tibble::tibble(
-    label = data[[subgroup]],
-    level = names_subgroup
-  )
-  data[[subgroup]] <- as.character(names_subgroup)
-  meta_subgroup <- dplyr::arrange(dplyr::distinct(all_subgroup), level)
-  attr(data, "brm_reference_subgroup") <- brm_levels(reference_subgroup)
-  attr(data, "brm_levels_subgroup") <- sort(as.character(meta_subgroup$level))
-  attr(data, "brm_labels_subgroup") <- sort(as.character(meta_subgroup$label))
-  data
-}
-
-brm_data_level_time <- function(data) {
-  time <- attr(data, "brm_time")
-  reference_time <- attr(data, "brm_reference_time")
-  names_time <- brm_levels(data[[time]])
-  all_time <- tibble::tibble(label = data[[time]], level = names_time)
-  data[[time]] <- as.character(names_time)
-  meta_time <- dplyr::arrange(dplyr::distinct(all_time), level)
-  if (!is.null(reference_time)) {
-    attr(data, "brm_reference_time") <- brm_levels(reference_time)
-  }
-  attr(data, "brm_levels_time") <- sort(as.character(meta_time$level))
-  attr(data, "brm_labels_time") <- sort(as.character(meta_time$label))
-  data
-}
-
-brm_levels <- function(x) {
-  x <- as.character(x)
-  out <- make.names(x, unique = FALSE, allow_ = TRUE)
-  assert(
-    all(duplicated(x) == duplicated(out)),
-    message = paste0(
-      "Levels collapsed while trying to create valid variable names. Input: ",
-      brm_chr_head_unique(x),
-      ". Output: ",
-      brm_chr_head_unique(out),
-      ". Please make sure the elements of the treatment group and discrete ",
-      "time columns of the data still indicate the correct groupings ",
-      "even after make.names(unique = FALSE, allow_ = TRUE)."
-    )
-  )
-  out
-}
-
-brm_chr_head_unique <- function(x, n = 30L) {
-  paste(utils::head(unique(x), n = n), collapse = ", ")
 }
 
 brm_data_fill <- function(data) {
@@ -471,12 +398,12 @@ brm_data_fill.brms_mmrm_data <- function(data) {
   group <- attr(data, "brm_group")
   subgroup <- attr(data, "brm_subgroup")
   time <- attr(data, "brm_time")
-  levels_time <- attr(data, "brm_levels_time")
   patient <- attr(data, "brm_patient")
   covariates <- attr(data, "brm_covariates")
   missing <- attr(data, "brm_missing")
   interest <- attr(data, "brm_archetype_interest")
   nuisance <- attr(data, "brm_archetype_nuisance")
+  data <- droplevels(data)
   args <- list(data = data, as.symbol(patient), as.symbol(time))
   data <- do.call(what = tidyr::complete, args = args)
   args <- list(.data = data, as.symbol(patient), as.symbol(time))
@@ -514,8 +441,9 @@ brm_data_fill.brms_mmrm_data <- function(data) {
   for (name in names(attributes)) {
     attr(out, name) <- attributes[[name]]
   }
+  times <- brm_levels(out[[time]])
   assert(
-    out[[time]] == rep(levels_time, times = nrow(data) / length(levels_time)),
+    length(unique(table(out[[time]]))) == 1L,
     message = paste(
       "data could not be filled. Please submit a bug report to",
       "https://github.com/openpharma/brms.mmrm/issues",
@@ -525,6 +453,19 @@ brm_data_fill.brms_mmrm_data <- function(data) {
   out
 }
 
+brm_time_contrasts <- function(data) {
+  time <- attr(data, "brm_time")
+  if (is.ordered(data[[time]])) {
+    n <- length(unique(data[[time]]))
+    contrasts(data[[time]]) <- stats::contr.treatment(n = n)
+  }
+  data
+}
+
+brm_levels <- function(x) {
+  if_any(is.factor(x), intersect(levels(x), unique(x)), sort(unique(x)))
+}
+
 brm_data_fill_column <- function(x, index) {
   out <- tapply(
     X = x,
